/*
 *  GOL - Game of life simulation software
 *  Copyright (C) 1996 Michael Raymond Brown
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Author: Michael Raymond Brown
 *  EMail:  michael@arrakeen.demon.co.uk
 *  WWW:    http://www.arrakeen.demon.co.uk/
 *  Address:  Michael Brown
 *            Kia Ora
 *            Brook Street
 *            Great Bromley
 *            Nr Colchester
 *            Essex
 *            CO7 7HX
 *            England
 */


#ifdef __GNUC__
#pragma implementation
#endif

#include "Image.h"


void xwImage::plot(int x, int y, int colour)
   {if (x>=0 && y>=0 && x<width && y<height)
	XPutPixel(ximage, x, y, colour);
   }
int xwImage::point(int x, int y)
   {return (x>=0 && y>=0 && x<width && y<height)
	? XGetPixel(ximage, x, y)
	: 0;
   }



void xwImage::putImage(int src_x, int src_y, int dst_x, int dst_y, int width, int height, xwDrawArea* area)
   {
#ifdef HAVE_MITSHM
    if (use_shm)
       {XShmPutImage(area->getDisplay(), area->getWindow(), area->getGC()
		, ximage, src_x, src_y, dst_x, dst_y, width, height, False);
	XSync(display, False);
       }
    else
#endif
    XPutImage(area->getDisplay(), area->getWindow(), area->getGC()
	, ximage, src_x, src_y, dst_x, dst_y, width, height);
   }
void xwImage::putImage(int x, int y, xwDrawArea* area)
   {putImage(0, 0, x, y, ximage->width, ximage->height, area);
   }



xwImage::xwImage()
   {
   }
xwImage::~xwImage()
   {if (ximage)
#ifdef HAVE_MITSHM
    if (use_shm)
       {shmdt(shminfo.shmaddr);
	XShmDetach(display, &shminfo);
	shmctl(shminfo.shmid, IPC_RMID, (struct shmid_ds*)0);
	ximage->data = (char*)0;
	XDestroyImage(ximage);
       }
    else
#endif
       {delete [] (unsigned int*)ximage->data;
	ximage->data = (char*)0;
	XDestroyImage(ximage);
   }   }


void xwImage::CreateImage(xwApp* app, int format, int depth)
   {unsigned int* data;
    char* ptr;
    int crap;

    display=app->getDisplay();
#ifdef HAVE_MITSHM
    if (app->noshm) goto skipit;
    ptr = DisplayString(display);
    if (  (!ptr || *ptr==':'
	|| !strncmp(ptr, "localhost:", 10)
	|| !strncmp(ptr, "unix:", 5)
	|| !strncmp(ptr, "local:", 6))
	&& XShmQueryVersion(display, &crap, &crap, &crap))
       {
	ximage = XShmCreateImage(display, DefaultVisual(display, DefaultScreen(display))
		, depth, format, (char*)0, &shminfo, width, height);
	shminfo.shmid = shmget(IPC_PRIVATE
		, ximage->bytes_per_line*(ximage->height+2)
		, IPC_CREAT|0777);
	if (shminfo.shmid == -1)
	   {XDestroyImage(ximage);
	    goto skipit;
	   }
	shminfo.shmaddr=ximage->data=shmat(shminfo.shmid, (char*)0, 0);
	shminfo.readOnly=True;
	if (!XShmAttach(display, &shminfo))
	   {shmdt(shminfo.shmaddr);
	     shmctl(shminfo.shmid, IPC_RMID, (struct shmid_ds*)0);
	    ximage->data=(char*)0;
	    XDestroyImage(ximage);
	    goto skipit;
	   }
	XSync(display, False);
	use_shm=1;
	return;
       }
skipit:
    use_shm=0;
#endif
       {
	ximage = XCreateImage(display, DefaultVisual(display, DefaultScreen(display))
		, depth, format, 0, (char*)0, width, height, 32, 0);
	data = new unsigned int[(ximage->bytes_per_line*(ximage->height+2)+sizeof(int)-1)/sizeof(int)];
	ximage->data = (char*)data;
   }   }


